//-------------------------------------------------------------------
// code03e8.c
//-------------------------------------------------------------------

#include <PalmOs.h>
#include <Silk.h>
#include <Vga.h>
#include <HostControl.h>

#include "ELaunch.h"
#include "Resource.h"

/*##################################################################*/
asm ( " bra MyEvtProcessSoftKeyStroke" );
/*##################################################################*/

typedef struct {
  Coord	DispHeight;
  Coord	DispWidth1_3;
  Coord	GraffitiWidth1_2;
  Coord	DispWidth2_3;
  Coord	LeftSilkX;
  Coord	RightSilkX;
  Coord	BetweenSilksHoriz;
  } TDisplayCoords;

const static TDisplayCoords Palm_DisplayCoords = 
{
  160,
  160 / 3,
  160 / 2,
  ( 160 * 2 ) / 3,
  26,
  160 - 26,
  191
};

const static TDisplayCoords HandEra_DisplayCoords1to1 = 
{
  320,
  240 / 3,
  240 / 2,
  ( 240 * 2 ) / 3,
  40,
  240 - 40,
  360
};

const static TDisplayCoords HandEra_DisplayCoordsToFit = 
{
  320,
  160 / 3,
  240 / 2,
  ( 160 * 2 ) / 3,
  40,
  240 - 40,
  360
};

/*##################################################################*/

int Point ( PointType* p, const TDisplayCoords* pDisplayCoords );

/*##################################################################*/

typedef struct
{
  const TDisplayCoords	*pDisplayCoords;
  UInt32		SilkVersion;
  int			iFrom;
  int			iTo;
  VgaScreenModeType	mode;
  VgaRotateModeType	rotation;
  TGlobals		*pVars;
  EventType		Event;
  TPrefs		Prefs;
  UInt16		wSize;
  TCommonVars		CommonVars;
} TVars;

/*##################################################################*/
Err MyEvtProcessSoftKeyStroke	( PointType* start, PointType* end ) 
/*##################################################################*/
{
  Err (*oldtrap)( PointType*, PointType* );
  TVars *vars = (TVars*)MemPtrNew( sizeof( *vars ) );

#ifdef TRACKME
  {
    const static char gszFormat[] = "EvtProcessSoftKeyStroke [%d:%d] > [%d:%d]";

    HostTraceInit();
    HostTraceOutputTL(appErrorClass, gszFormat, start->x, start->y, end->x, end->y );
  }
#endif

  if ( NULL != vars )
  {
    Boolean Handled = false;

    if ( _TRGSilkFeaturePresent( &(vars->SilkVersion) ) )
    {
      // HandEra 330

      if ( SilkWindowMaximized() )
      {
        VgaGetScreenMode( &(vars->mode), &(vars->rotation) );

        vars->pDisplayCoords = vars->mode == screenModeScaleToFit ? &HandEra_DisplayCoordsToFit : &HandEra_DisplayCoords1to1;
      }
      else
        vars->pDisplayCoords = NULL;
    }
    else
      // Palm 160x160
      vars->pDisplayCoords = &Palm_DisplayCoords;

    vars->iFrom = Point( start, vars->pDisplayCoords );

    if ( vars->iFrom < FROMCOUNT )
    {
      vars->iTo = Point( end, vars->pDisplayCoords );

      if ( vars->iTo < TOCOUNT )
      {
        vars->pVars = NULL;
        FtrGet( HACKID, FTR_VARS, (UInt32*)&(vars->pVars) );

        if ( NULL != vars->pVars )
        {
          vars->Event.eType = EVENT_STROKE;
          vars->Event.data.generic.datum[0] = vars->iFrom;
          vars->Event.data.generic.datum[1] = vars->iTo;
          EvtAddEventToQueue( &(vars->Event) );
	  Handled = true;
        }
        else
        {
          vars->wSize = sizeof( vars->Prefs );

          if ( APPPREFVER == PrefGetAppPreferences( HACKID, APPPREFID, &(vars->Prefs), &(vars->wSize), true ) )
            if ( DoAnyActionProxy( vars->Prefs.aStrokes[ vars->iTo * FROMCOUNT + vars->iFrom ], &(vars->Prefs), &(vars->CommonVars) ) )
	      Handled = true;
        }
      }
    }

    MemPtrFree( vars );

    if ( Handled )
      return 0;
  }

  FtrGet( HACKID, HACKINDEX1, (UInt32*)&oldtrap );
  return oldtrap( start, end );
}
/*##################################################################*/
int Point ( PointType* p, const TDisplayCoords* pDisplayCoords )
/*##################################################################*/
{
  if ( NULL == pDisplayCoords )
  {
    if ( p->y > 385 )
      return p->x < 20 ?  aAPPS :
	     p->x < 40 ?  aMENU :
	     p->x < 220 ? aCALC : aFIND;
    else
      return TOCOUNT;
  }
  else
    if ( p->y < pDisplayCoords->DispHeight )
      return p->x > pDisplayCoords->DispWidth2_3 ? aDISPRIGHT :
	     p->x > pDisplayCoords->DispWidth1_3 ? aDISPMIDDLE : aDISPLEFT ;
    else
    {
      int Horizontal = 0;

      if ( p->x >= pDisplayCoords->LeftSilkX )
      {
        Horizontal++;

        if ( p->x >= pDisplayCoords->GraffitiWidth1_2 )
        {
          Horizontal++;

	if ( p->x >= pDisplayCoords->RightSilkX )
	  Horizontal++;
        }
      }

      return p->y > pDisplayCoords->BetweenSilksHoriz ? aMENU + Horizontal : aAPPS + Horizontal;
    }
}
/*##################################################################*/
#include "common.c"
/*##################################################################*/
