//-------------------------------------------------------------------
// code0bb8.c
//-------------------------------------------------------------------

#include <PalmOs.h>
#include <Audio.h>
#include <AutoCF.h>
#include <Silk.h>
#include <HostControl.h>

#include "ELaunch.h"
#include "Resource.h"

/*##################################################################*/

asm ( " bra DoAnyAction" );

/*##################################################################*/

const static Char gszIrDaLibrary[] = "IrDA Library";

const static Char gszValume[] = "Volume";

const static UInt16 gaEvtChars[] = {
     backlightChr,
     hardPowerChr,
     lockChr,
     keyboardChr,
     graffitiReferenceChr,
     sendDataChr,
     hardCradleChr,
     contrastChr,
     brightnessChr,
     findChr,
     pageUpChr,
     pageDownChr,
     autoOffChr,
     menuChr,
     escapeChr,
     commandChr
};

/*##################################################################*/

typedef struct
{
  FieldPtr	      pCurField;
  int		      iFieldInsPos;
  int		      iFieldTextLen;
  UInt16	      wSelStart;
  UInt16	      wSelEnd;
  UInt16	      wEdge;
  FormPtr	      pCurForm;
  UInt16	      uiCard;
  LocalID	      DBid;
  DmSearchStateType   ss;
  UInt32	      Version;
  Boolean	      OS3;
  Boolean	      TRGaudio;
  Boolean             TRGsilk;
  int		      Index;
  int		      PrefSoundStart;
  int		      SoundTypesCount;
  int		      PrefChoice;
  char		      *pszText;
  UInt16	      LevelSum;
  UInt16	      OffVolume;
  Boolean	      SoundWasOff;
  Boolean	      SoundNew;
  UInt16	      MaxAmp;
  UInt16	      MinAmp;
  UInt16	      IrLibRef;
  Boolean	      ScanningOn;
  UInt16	      ScanningOnLen;
  Boolean	      UnMute;
  Boolean	      AutoCfState;
  char		      *Buffer;
  DmOpenRef	      db;
  UInt16	      open_count;
  MemHandle	      CodeHandle;
  void		      (*CodePointer)();
  Boolean	      InsPtWasEnabled;
  UInt32	      saveStack;
  UInt32	      newStack;
  UInt16	      FormId;
  WinHandle	      DrawWindow;
  WinHandle	      ActiveWindow;
  FormPtr	      OldForm;
  LocalID	      CurLocalID;
  UInt16	      CurCardOn;
  SndCommandType      sndCmd;
} TVars;

/*##################################################################*/
void  SetSelection  ( TVars* pVars )
/*##################################################################*/
{
  FldSetInsPtPosition( pVars->pCurField, pVars->iFieldInsPos );

  if ( pVars->iFieldInsPos > pVars->wEdge )
    FldSetSelection( pVars->pCurField, pVars->wEdge, pVars->iFieldInsPos );
  else
    FldSetSelection( pVars->pCurField, pVars->iFieldInsPos, pVars->wEdge );
}

/*##################################################################*/
void  SetCursor ( TVars* pVars, int iPos )
/*##################################################################*/
{
  pVars->iFieldInsPos = pVars->wEdge = iPos;
  SetSelection( pVars );
}

/*##################################################################*/
Boolean	GetCurField	( TVars*	pVars )
/*##################################################################*/
{
  UInt16	wCurFocus;
  void		*pObject;

  if ( NULL == ( pVars->pCurForm = FrmGetActiveForm() ) )
    return false;

  if ( -1 == ( wCurFocus = FrmGetFocus( pVars->pCurForm ) ) )
    return false;

  if ( NULL == ( pObject = FrmGetObjectPtr( pVars->pCurForm, wCurFocus ) ) )
    return false;

  switch( FrmGetObjectType( pVars->pCurForm, wCurFocus ) )
    {
    case frmFieldObj:
      pVars->pCurField = (FieldPtr)pObject;
      break;

    case frmTableObj:
      if ( NULL == ( pVars->pCurField = TblGetCurrentField( (TablePtr)pObject ) ) )
        return false;
      break;

    default:
      return false;
    }

  if ( NULL == ( pVars->pszText = FldGetTextPtr( pVars->pCurField ) ) )
    return false;

  pVars->iFieldTextLen = FldGetTextLength( pVars->pCurField );
  FldGetSelection( pVars->pCurField, &(pVars->wSelStart), &(pVars->wSelEnd) );
  pVars->iFieldInsPos  = FldGetInsPtPosition( pVars->pCurField );

  pVars->wEdge = pVars->iFieldInsPos == pVars->wSelStart ? pVars->wSelEnd : pVars->wSelStart;

  return true;
}

/*##################################################################*/
void ToggleTone ( Boolean OnOff, TPrefs* pPrefs, TVars *vars )
/*##################################################################*/
{
  if ( pPrefs->CheckBoxes[ cToggleIndication ] )
  {
    vars->sndCmd.cmd    = sndCmdFreqDurationAmp;
    vars->sndCmd.param1 = OnOff ? TONE_HIGH : TONE_LOW;
    vars->sndCmd.param2 = TONE_DURATION;
    vars->sndCmd.param3 = PrefGetPreference( prefSysSoundVolume );

    SndDoCmd( 0, &(vars->sndCmd), true );
  }
}

/*##################################################################*/
int	FindLine	( TVars *vars, 
			  Boolean fBegin )
/*##################################################################*/
{
  UInt16	w1, w2, w3;
  int	i;

  if ( NULL == ( vars->pCurField->lines ) )
    return fBegin ? 0 : vars->iFieldTextLen;

  FldGetScrollValues( vars->pCurField, &w1, &w2, &w3 );

  for ( i = 0 ; i < w2 ; i++ )
  {
    LineInfoPtr	p = vars->pCurField->lines + i;
    UInt16	wTemp;
    
    if ( ( wTemp = ( p->length + ( i == w2 - 1 ) ) + p->start - 1 ) >= vars->iFieldInsPos )
    {
      if ( fBegin )
        return p->start;
      else
        if ( wTemp > vars->iFieldTextLen )
          return vars->iFieldTextLen;
        else
          return wTemp;
    }
  }

  return -1;
}

/*##################################################################*/
Boolean DoAnyAction ( UInt32 dwCode, TPrefs* pPrefs )
/*##################################################################*/
{
  Boolean Result = false;

  TVars *vars = (TVars*)MemPtrNew( sizeof( *vars ) );

  if ( NULL != vars )
  {
    FtrGet( sysFtrCreator, sysFtrNumROMVersion, &(vars->Version) );
    vars->OS3 = vars->Version >= 0x03000000;

    vars->TRGaudio = FtrGet( TRGSysFtrID, TRGAudioFtrNum, &(vars->Version) ) == 0;
    vars->TRGsilk = FtrGet( TRGSysFtrID, TRGSilkFtrNum, &(vars->Version) ) == 0;

    if ( dwCode < 0x100 )
    {
      if ( pPrefs->CheckBoxes[ cClickOnOther ] )
        SndPlaySystemSound( sndClick );

      switch( dwCode )
      {
        case oCUT + 1:
          if ( GetCurField( vars ) )
            FldCut( vars->pCurField ); 	
          break;

        case oCOPY + 1:
          if ( GetCurField( vars ) )
            FldCopy( vars->pCurField ); 	
          break;

        case oPASTE + 1:
          if ( GetCurField( vars ) )
            FldPaste( vars->pCurField ); 	
          break;

        case oUNDO + 1:
          if ( GetCurField( vars ) )
            FldUndo( vars->pCurField ); 	
          break;

        case oSELECTALL + 1:
          if ( GetCurField( vars ) )
            FldSetSelection( vars->pCurField, 0, FldGetTextLength( vars->pCurField ) );
          break;

        case oTOGGLESOUND + 1:
        case oSOUNDON + 1:
        case oSOUNDOFF + 1:
        {
          vars->MaxAmp = sndMaxAmp;

          if ( vars->OS3 )
          {
            vars->PrefSoundStart = prefSysSoundVolume;
            vars->OffVolume = 0;
          }
          else
          {
            vars->PrefSoundStart = prefSysSoundLevelV20;
            vars->OffVolume = slOff;
          }

  	for ( vars->Index = vars->SoundTypesCount = pPrefs->CheckBoxes[ cKeepAlarm ] ? SOUNDTYPES - 1 : SOUNDTYPES, vars->LevelSum = 0 ; 
  	      --(vars->Index) >= 0 ; )
            vars->LevelSum += PrefGetPreference( vars->PrefSoundStart + vars->Index );

          vars->SoundWasOff = vars->LevelSum == ( vars->SoundTypesCount * vars->OffVolume );

  	if ( dwCode == oTOGGLESOUND + 1 )
  	  vars->SoundNew = vars->SoundWasOff;
  	else
  	  vars->SoundNew = dwCode == oSOUNDON + 1;

          if ( !vars->SoundWasOff )
            ToggleTone( vars->SoundNew, pPrefs, vars );

          for ( vars->Index = SOUNDTYPES ; --(vars->Index) >= 0 ; )
          {
            vars->PrefChoice = vars->PrefSoundStart + vars->Index;

            if ( !vars->SoundWasOff )
              pPrefs->SoundLevels[ vars->Index ] = PrefGetPreference( vars->PrefChoice );

  	  if ( vars->Index < vars->SoundTypesCount )
  	    PrefSetPreference( vars->PrefChoice, vars->SoundNew ? pPrefs->SoundLevels[ vars->Index ] : vars->OffVolume );
          }

          if ( vars->SoundNew )
          {
            SndSetDefaultVolume( pPrefs->SoundLevels + ( prefAlarmSoundVolume - prefSysSoundVolume),
        			 pPrefs->SoundLevels + 0,
        			 &(vars->MaxAmp) );
          }
          else
          {
            vars->MinAmp = vars->OffVolume;

            PrefSetAppPreferences ( HACKID, APPPREFID, APPPREFVER,
            			    pPrefs, sizeof( *pPrefs ), true );

  	    SndSetDefaultVolume( pPrefs->CheckBoxes[ cKeepAlarm ] ? pPrefs->SoundLevels + ( prefAlarmSoundVolume - prefSysSoundVolume) : &(vars->MinAmp), 
  				  &(vars->MinAmp), 
  				  &(vars->MaxAmp) );

          }

          if ( vars->SoundWasOff )
            ToggleTone( vars->SoundNew, pPrefs, vars );

          break;
        }

        case oBACKLIGHT_INVERT + 1:
  	  EvtEnqueueKey( backlightChr, 0, commandKeyMask );
	  // fall down

        case oINVERT + 1:
        {
          *((UInt8*)0xfffffa21) ^= 1;
          break;
        }

        case oTOGGLEBEAMRECEIVE + 1:
        case oBEAMRECEIVEON + 1:
        case oBEAMRECEIVEOFF + 1:
        {
          if ( !SysLibFind( (Char*)gszIrDaLibrary, &(vars->IrLibRef) ) )
          {
            vars->ScanningOnLen = sizeof( vars->ScanningOn );

            if ( !ExgLibControl( vars->IrLibRef, irGetScanningMode, &(vars->ScanningOn), &(vars->ScanningOnLen) ) )
            {
  	      if ( dwCode == oTOGGLEBEAMRECEIVE + 1 )
  	        vars->ScanningOn ^= true;
  	      else
  	        vars->ScanningOn = dwCode == oBEAMRECEIVEON + 1;

              ExgLibControl( vars->IrLibRef, irSetScanningMode, &(vars->ScanningOn), &(vars->ScanningOnLen) );

              ToggleTone( vars->ScanningOn, pPrefs, vars );
            }
          }
        break;
        }

        case oTOGGLERECORDS + 1:
  	  if ( PrefGetPreference( prefHidePrivateRecordsV33 ) )
  	    goto oSHOWRECORDSlabel;
	    // fall down

        case oHIDERECORDS + 1:
  	  PrefSetPreference( prefHidePrivateRecordsV33, true );
  	  ToggleTone( false, pPrefs, vars );
  	  break;

        case oSHOWRECORDS + 1:
        oSHOWRECORDSlabel:
  	  PrefSetPreference( prefHidePrivateRecordsV33, false );
  	  ToggleTone( true, pPrefs, vars );
  	  break;

        case oMASKRECORDS + 1:
          PrefSetPreference( prefShowPrivateRecords, maskPrivateRecords );
          break;

        case oLASTAPP + 1:
        LASTAPPlabel:
          SysUIAppSwitch( pPrefs->LnchCardNo1, pPrefs->LnchDbID1, sysAppLaunchCmdNormalLaunch, 0 );
     	  break;

        case oRESET + 1:
          SysReset();
     	break;

        case oMUTEON + 1: 
        case oMUTEOFF + 1: 
        case oTOGGLEMUTE + 1:
  	  if ( vars->TRGaudio )
  	  {
  	    if ( dwCode == oTOGGLEMUTE + 1 )
  	      AudioGetMute( &(vars->UnMute) );
  	    else
  	      vars->UnMute = dwCode == oMUTEOFF + 1;

  	    AudioSetMute( vars->UnMute = !vars->UnMute );
  	    ToggleTone( vars->UnMute, pPrefs, vars );
  	  }
  	break;

        case oAUTOCFON + 1:
        case oAUTOCFOFF + 1:
        case oTOGGLEAUTOCF + 1:
  	  if ( FtrGet( TRG_CREATOR, AUTOCF_FEATURE_ID, &(vars->Version) ) == 0 && AUTOCF_LOADED )
  	  {
  	    if ( dwCode == oTOGGLEAUTOCF + 1 )
  	      vars->AutoCfState = !AcfIsEnabled();
  	    else
  	      vars->AutoCfState = dwCode == oAUTOCFON + 1;

  	    if ( vars->AutoCfState )
  	    {
  	      if ( NULL != ( vars->Buffer = MemPtrNew( 128 ) ) )
  	      {
  	        AcfGetPath( vars->Buffer );
  	        vars->AutoCfState = AcfEnable( vars->Buffer );
  	        MemPtrFree( vars->Buffer );
  	      }
  	    }
  	    else
  	      AcfDisable();

  	    ToggleTone( vars->AutoCfState, pPrefs, vars );
  	  }
  	  break;

        case oVOLUMEDLG + 1: 
  	  if ( vars->TRGaudio )
  	    AudioVolumeDlg( (char*)gszValume );
  	  break;

  /*      case oHOTSYNCLOCAL+1:*/
        case oHOTSYNCREMOTE+1:
  	  if ( !DmGetNextDatabaseByTypeCreator( true, &(vars->ss), sysFileTApplication, sysFileCSync, true, &(vars->uiCard), &(vars->DBid) ) )
  	    SysUIAppSwitch( vars->uiCard, vars->DBid, /*dwCode == oHOTSYNCLOCAL + 1 ? sysAppLaunchCmdSyncRequestLocal : */sysAppLaunchCmdSyncRequestRemote, NULL );

  	break;

        case oEND+1:
	case oHOME+1:
          if ( GetCurField( vars ) )
	    SetCursor( vars, oHOME+1 == dwCode ? 0 : vars->iFieldTextLen );
  	break;

        case oLINE_END+1:
	case oLINE_HOME+1:
	  if ( GetCurField( vars ) )
	  {
	    int	iPos;

	    if ( -1 != ( iPos = FindLine( vars, oLINE_HOME+1 == dwCode ) ) )
	      SetCursor( vars, iPos );
	  }
  	break;

        case oGRAFFITION+1:
          if ( vars->TRGsilk )
            SilkMaximizeWindow();
          break;

        case oGRAFFITIOFF+1:
          if ( vars->TRGsilk )
            SilkMinimizeWindow();
          break;

        case oTOGGLEGRAFFITI+1:
          if ( vars->TRGsilk )
          {
            if ( SilkWindowMaximized() )
              SilkMinimizeWindow();
            else
              SilkMaximizeWindow();
          }
          break;

        default:  
        {
          EvtEnqueueKey( gaEvtChars[ dwCode - 1 ], 0, commandKeyMask );
          break;
        }
      }

      Result = true;
    }
    else
    {
      if ( 0 == DmGetNextDatabaseByTypeCreator( true,
    						&(vars->ss),

    						( dwCode & DACREATORFLAG ) ? TYPE_DA : 
  						( dwCode & PANLCREATORFLAG ) ? TYPE_PANL : TYPE_APP,

    						dwCode & ~( ALLCREATORFLAGS ),
    						true,
    						&(vars->uiCard),
    						&(vars->DBid) ) )
      {
        if ( pPrefs->CheckBoxes[ cClickOnApp ] )
    	  SndPlaySystemSound( sndClick );

        if ( dwCode & DACREATORFLAG )
        {
          vars->db = DmOpenDatabase( vars->uiCard, vars->DBid, dmModeReadOnly );

          if ( NULL != vars->db )
          {
    	    if ( 0 == DmOpenDatabaseInfo( vars->db, NULL, &(vars->open_count), NULL, NULL, NULL ) )
    	    {
    	      if ( 1 == vars->open_count )
    	      {
    	        vars->CodeHandle = DmGet1Resource( 'code', 1000 ) ;

    	        if ( NULL != vars->CodeHandle )
    	        {
    	          vars->CodePointer = MemHandleLock( vars->CodeHandle );

    	          if ( NULL != vars->CodePointer )
    	          {
    	    	    vars->newStack = 0;

    	    	    if ( !vars->OS3 )
    	    	    {
    	    	      vars->newStack= (UInt32)MemPtrNew( DASTACKSIZE ) ;
    	    	      if ( vars->newStack != 0 )
    	    	      {
    	    	        vars->saveStack= __getA7();
    	    	        __setA7( (long)vars->newStack + DASTACKSIZE );
    	    	      }
    	    	    }

    		    EvtFlushNextPenStroke();

    		    vars->FormId = FrmGetActiveFormID();
    		    vars->OldForm = FrmGetActiveForm();

    		    vars->ActiveWindow = WinGetActiveWindow();
    		    vars->DrawWindow = WinGetDrawWindow();

    	    	    vars->InsPtWasEnabled = InsPtEnabled();
    	    	    InsPtEnable( false );

    	    	    vars->CodePointer();

    	    	    InsPtEnable( vars->InsPtWasEnabled );

    		    if ( vars->FormId != 0 )
    		      FrmSetActiveForm( vars->OldForm );

    		    WinSetActiveWindow( vars->ActiveWindow );
    		    WinSetDrawWindow( vars->DrawWindow );		

    	    	    if ( vars->newStack != 0 )
    	    	    {
    	    	      __setA7( vars->saveStack );
    	    	      MemPtrFree( (MemPtr)vars->newStack );
    	    	    }

    	    	    MemHandleUnlock( vars->CodeHandle );
    	          }
    
    	          DmReleaseResource( vars->CodeHandle );
    	        }
    	      }
    	    }

    	    DmCloseDatabase( vars->db );
          }
        }
        else
        {
  	  if ( 0 != SysCurAppDatabase( &(vars->CurCardOn), &(vars->CurLocalID) ) ||
  	       vars->CurCardOn != vars->uiCard ||
  	       vars->CurLocalID != vars->DBid )
  	    SysUIAppSwitch( vars->uiCard, vars->DBid, sysAppLaunchCmdNormalLaunch, 0 );
  	  else
  	    if ( pPrefs->CheckBoxes[ cLaunchLast ] )
  	      goto LASTAPPlabel;
	  }

        Result = true;
      }
    }

    MemPtrFree( vars );
  }

  return Result;
}
/*##################################################################*/
